
#ifdef GL_ES
precision mediump float;
#endif

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform highp float size; //swift 객체에서 값을 조절 할땐 값이 설정 되면 변경이 안됨 (선언만 하고)


vec3 channel_mix(vec3 a, vec3 b, vec3 w) {
    return vec3(mix(a.r, b.r, w.r), mix(a.g, b.g, w.g), mix(a.b, b.b, w.b));
}

float gaussian(float z, float u, float o) {
    return (1.0 / (o * sqrt(2.0 * 3.1415))) * exp(-(((z - u) * (z - u)) / (2.0 * (o * o))));
}

vec3 madd(vec3 a, vec3 b, float w) {
    return a + a * b * w;
}

vec3 screen(vec3 a, vec3 b, float w) {
    return mix(a, vec3(1.0) - (vec3(1.0) - a) * (vec3(1.0) - b), w);
}

vec3 overlay(vec3 a, vec3 b, float w) {
    return mix(a, channel_mix(
                              2.0 * a * b,
                              vec3(1.0) - 2.0 * (vec3(1.0) - a) * (vec3(1.0) - b),
                              step(vec3(0.5), a)
                              ), w);
}

vec3 soft_light(vec3 a, vec3 b, float w) {
    return mix(a, pow(a, pow(vec3(2.0), 2.0 * (vec3(0.5) - b))), w); //softlight
}


//http://byteblacksmith.com/improvements-to-the-canonical-one-liner-glsl-rand-for-opengl-es-2-0/
highp float rand(vec2 co)
{
    highp float a = 12.9898;
    highp float b = 78.233;
    highp float c = 43758.5453;
    highp float dt= dot(co.xy ,vec2(a,b));
    highp float sn= mod(dt,3.14);
    return fract(sin(sn) * c);
}


void main() {
    float rnd = rand( textureCoordinate );
    lowp vec4 color = texture2D(inputImageTexture, textureCoordinate);
    lowp vec3 grain = vec3(rnd) * (1.0 - color.rgb);
    //lowp vec3 grain = vec3(rnd);
    
    //color.rgb += grain * size;
    //color.rgb = overlay(color.rgb, grain, size);
    
    //color.rgb = screen(color.rgb, grain, w);
    //color.rgb = max(color.rgb, grain * w);
    
    //gl_FragColor = vec4(vec3(color),1.0);
    //gl_FragColor = vec4(grain,1.0);
    
    //color.rgb = soft_light(color.rgb, grain, size);
    color.rgb = soft_light(color.rgb, grain, size);
   
    
    //color.rgb = overlay(color.rgb, grain, size);
    
    gl_FragColor = color;
}




//https://www.shadertoy.com/view/4t2fRz
void main2()
{
    
    //lowp vec2 ps = vec2(1.0) / vec2(300,400);
    lowp vec2 uv = textureCoordinate / vec2(1000.0,1000.0);
    
    lowp vec4 color = texture2D(inputImageTexture, textureCoordinate);
    //color = texture(inputImageTexture, uv);
    
    lowp float seed = dot(uv, vec2(12.9898,78.233));
    lowp float noise = fract(sin(seed) * 43758.5453123);
    
    lowp float w = 0.075;
    lowp vec3 grain = vec3(noise) * (1.0 - color.rgb);
    
    color.rgb += grain * w;
    //color.rgb = vec3(noise);
    //color.rgb = soft_light(color.rgb, grain, w);
    
    gl_FragColor = color;
    
}


//https://www.shadertoy.com/view/4sXSWs



float random (vec2 st) {
    return fract(sin(dot(st.xy,
                         vec2(12.9898,78.233)))*
                 43758.5453123);
}


float randomNoise(vec2 p) {
    return fract(6791.*sin(47.*p.x+p.y*9973.));
}

/*
 void main2() {
 
 lowp vec2 position = textureCoordinate.xy/vec2(670,894);
 lowp float n = randomNoise(position);
 gl_FragColor = vec4(vec3(n), 1.);
 
 }*/



/*
 void mainImage(out vec4 color, vec2 coord) {
 vec2 ps = vec2(1.0) / iResolution.xy;
 vec2 uv = coord * ps;
 
 color = texture(iChannel0, uv);
 
 float t = 1.0;
 float seed = dot(uv, vec2(12.9898, 78.233));
 float noise = fract(sin(seed) * 43758.5453 + t);
 
 float w = 0.1;
 
 vec3 grain = vec3(noise) * (1.0 - color.rgb);
 color.rgb += grain * w;
 
 }
 */

