precision highp float;
varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

const vec4  kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const vec4  kRGBToI     = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const vec4  kRGBToQ     = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const vec4  kYIQToR   = vec4 (1.0, 0.9563, 0.6210, 0.0);
const vec4  kYIQToG   = vec4 (1.0, -0.2721, -0.6474, 0.0);
const vec4  kYIQToB   = vec4 (1.0, -1.1070, 1.7046, 0.0);
uniform float hueAdjust;
void main ()
{
    lowp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
    //float hueAdjust = 10.6;//9.6
    // Sample the input pixel
    vec4 color   = texture2D(inputImageTexture, textureCoordinate);
    
    // Convert to YIQ
    float   YPrime  = dot (color, kRGBToYPrime);
    float   I      = dot (color, kRGBToI);
    float   Q      = dot (color, kRGBToQ);
    
    // Calculate the hue and chroma
    float   hue     = atan (Q, I);
    float   chroma  = sqrt (I * I + Q * Q);
    
    // Make the user's adjustments
    hue += (-hueAdjust); //why negative rotation?
    
    // Convert back to YIQ
    Q = chroma * sin (hue);
    I = chroma * cos (hue);
    
    // Convert back to RGB
    vec4 yIQ = vec4 (YPrime, I, Q, 0.0);
    color.r = dot (yIQ, kYIQToR);
    color.g = dot (yIQ, kYIQToG);
    color.b = dot (yIQ, kYIQToB);
    
    //float ratio = textureColor.g * (1.0 - (textureColor.r * textureColor.b));
    
    //float ratio = textureColor.g;
    //ratio = ratio * (textureColor.r * textureColor.b);
    
    float ratio = 0.0;
    ratio = sqrt(pow(0.0 - textureColor.r ,2.0) + pow(1.0 - textureColor.g,2.0) + pow(0.0 - textureColor.b,2.0));
    
    ratio = 1.0 / ratio;
    ratio = pow(ratio * 0.7, 5.0);
    
    if(1.0 < ratio){
        ratio = 1.0;
    }
    
    if(ratio < 0.0){
        ratio = 0.0;
    }
    
    
    gl_FragColor = mix(textureColor, color, ratio);
    
    //gl_FragColor = color;
}





/*
 varying highp vec2 textureCoordinate;
 
 uniform sampler2D inputImageTexture;
 
 void main()
 {
 lowp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
 lowp vec4 outputColor;
 
 if(0.5 < textureColor.g && textureColor.b < 0.5 && textureColor.r < 0.5){
 //outputColor.r = 0.91 * textureColor.r;
 //outputColor.g = 0.76 * textureColor.g;
 //outputColor.b = 0.85 * textureColor.b;
 
 outputColor.g = 1.0 - textureColor.g;
 outputColor.r = textureColor.r;
 outputColor.b = textureColor.b;
 outputColor.a = 1.0;
 
 gl_FragColor = outputColor;
 }else{
 gl_FragColor = textureColor;
 }
 
 }*/


/*
 precision highp float;
 
 varying highp vec2 textureCoordinate;
 varying highp vec2 textureCoordinate2;
 
 
 uniform sampler2D inputImageTexture;
 
 void main()
 {
 float thresholdSensitivity = 0.4;
 float smoothing = 0.2;
 vec3 colorToReplace = vec3(0.0,1.0,0.0);
 
 vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
 vec4 textureColor2 = vec4(0.91,0.76,0.85,1.0);
 
 float maskY = 0.2989 * colorToReplace.r + 0.5866 * colorToReplace.g + 0.1145 * colorToReplace.b;
 float maskCr = 0.7132 * (colorToReplace.r - maskY);
 float maskCb = 0.5647 * (colorToReplace.b - maskY);
 
 float Y = 0.2989 * textureColor.r + 0.5866 * textureColor.g + 0.1145 * textureColor.b;
 float Cr = 0.7132 * (textureColor.r - Y);
 float Cb = 0.5647 * (textureColor.b - Y);
 
 //     float blendValue = 1.0 - smoothstep(thresholdSensitivity - smoothing, thresholdSensitivity , abs(Cr - maskCr) + abs(Cb - maskCb));
 float blendValue = 1.0 - smoothstep(thresholdSensitivity, thresholdSensitivity + smoothing, distance(vec2(Cr, Cb), vec2(maskCr, maskCb)));
 
 if(0.7 < textureColor.g){
 gl_FragColor = mix(textureColor, textureColor2, blendValue);
 }else{
 gl_FragColor = textureColor;
 }
 
 }*/
